from datetime import datetime
import numpy as np


def load_bad_ips_dates(filename):
    with open(filename) as file:
        data = [[int(x) for x in line.split()] for line in file.readlines()]
        exclude_dates = dict()
        for d in data:
            date = datetime(*d[:5])
            exclude_dates[date] = [d[5], d[6], d[8] > 0]
    return exclude_dates


def load_red(filename):
    with open(filename) as file:
        lines = [x.strip() for x in file.readlines()]

    fmin_d = dict()
    fm2_d = dict()
    he_d = dict()
    foe_d = dict()
    hes_d = dict()
    estype_d = dict()
    foes_d = dict()
    fbes_d = dict()
    hf_d = dict()
    fof1_d = dict()
    hf2_d = dict()
    fof2_d = dict()
    fxi_d = dict()
    m3000f1_d = dict()
    m3000f2_d = dict()
    for line in lines:
        year = 2000 + int(line[:2])
        month = int(line[2:4])
        day = int(line[4:6])
        hour = int(line[6:8])
        minute = int(line[8:10])
        date = datetime(year, month, day, hour, minute)
        fmin = line[10:13].strip()
        fm2 = line[15:18].strip()
        he = line[20:23].strip()
        foe = line[25:28].strip()
        hes = line[30:33].strip()
        estype = line[35:37].strip()
        foes = line[40:43].strip()
        fbes = line[45:48].strip()
        hf = line[50:53].strip()
        fof1 = line[55:58].strip()
        hf2 = line[60:63].strip()
        fof2 = line[65:68].strip()
        fxi = line[70:73].strip()
        m3000f1 = line[75:78].strip()
        m3000f2 = line[80:83].strip()

        fof2_flag = line[68:70].strip()

        skip = ['A', 'B', 'C', 'E', 'F', 'G', 'S', 'R', 'Y', '-']

        bad_ips_dates_filename = 'data/IPS42_stat_m{:02d}.txt'.format(month)
        bad_ips_dates = load_bad_ips_dates(bad_ips_dates_filename)

        if fmin not in skip:
            fmin_d[date] = float(fmin)/10.0       

        if fof2 not in skip and not fof2_flag and date not in bad_ips_dates:
            fof2_d[date] = float(fof2)/10.0
   
        if hf not in skip:
            hf_d[date] = float(hf)

        if hes not in skip:
            hes_d[date] = float(hes)

        if foes not in skip:
            foes_d[date] = float(foes)/10.0

        if foe not in skip:
            foe_d[date] = float(foe)/100.0

        if he not in skip:
            he_d[date] = float(he)

    data = {'fmin': fmin_d, 'fof2': fof2_d, 
            'hf': hf_d, 'hes': hes_d,
            'foes': foes_d, 'foe': foe_d,
            'he': he_d}

    return data


if __name__ == '__main__':

    pairs = ( ('j6njan21_USRP.red', 'j6njan21_IPS42.red'),
              ('j6nfeb21_USRP.red', 'j6nfeb21_IPS42.red'),
              ('j6nmar21_USRP.red', 'j6nmar21_IPS42.red'),
              ('j6napr21_usrp.red', 'j6napr21_IPS-42.red'),
              ('j6nmay21_usrp.red', 'j6nmay21_IPS-42.red'),
              ('j6njun21_usrp.red', 'j6njun21_IPS-42.red'),
              ('j6njly21_usrp.red', 'j6njly21_IPS-42.red'),
              ('j6naug21_usrp.red', 'j6naug21_IPS-42.red'),
              ('j6nsep21_usrp.red', 'j6nsep21_IPS-42.red'),
              ('j6noct21_usrp.red', 'j6noct21_IPS-42.red'),
              ('j6nnov21_usrp.red', 'j6nnov21_IPS-42.red'),
              ('j6ndec21_usrp.red', 'j6ndec21_IPS-42.red')
    )

    with open('stat_Dfof2_monthly.txt', 'wt') as file:
        pass

    directory = 'data/'
    for m, pair in enumerate(pairs):
        usrp_filename = pair[0]
        ips_filename = pair[1]
        usrp_data = load_red(directory + usrp_filename)
        ips_data = load_red(directory + ips_filename)

        vals_usrp = usrp_data['fof2']
        vals_ips = ips_data['fof2']

        diffs_all = []
        diffs = [[] for i in range(24)]
        for val_usrp in vals_usrp:
            if val_usrp in vals_ips:
                diff = vals_usrp[val_usrp] - vals_ips[val_usrp]
                if abs(diff) > 0.5:
                    print(val_usrp, diff, vals_usrp[val_usrp], vals_ips[val_usrp])

                if abs(diff) < 1.05:
                    diffs_all.append(diff)
                    hour = val_usrp.hour
                    diffs[hour].append(diff)

        freqs = np.linspace(-2, 2, 41)

        hours = np.append(np.zeros((1,)), np.linspace(0, 23, 24), 0)
        data_fof2 = []
        for diff in diffs:
            hist = np.histogram(diff, freqs)
            num = np.sum(hist[0])

            data_fof2.append([h/num if num > 0 else 0 for h in hist[0]])

        data_fof2 = np.array(data_fof2)

        data_fof2 = np.insert(data_fof2, 0, freqs[:-1], 0)
        data_fof2 = np.insert(data_fof2, 0, hours, 1)

        hist = np.histogram(diffs_all, freqs)
        num = np.sum(hist[0])
        with open('Dfof2_{:02d}.txt'.format(m+1), 'wt') as file:
            for v, f in zip(hist[0], hist[1]):
                file.write('{:5.1f} {:8.4f} {:8d}\n'.format(f, v/num, v))

        with open('stat_Dfof2_monthly.txt', 'at') as file:
                file.write( '{:2d} {:3d} {:5.2f} {:5.2f} {:5.2f} {:5.2f}\n'.format(m+1, 
                                                                                   num, 
                                                                                   np.mean(diffs_all), 
                                                                                   np.std(diffs_all), 
                                                                                   np.max(diffs_all), 
                                                                                   np.min(diffs_all))
                                                                                   )
